/***************************************************************************
 *   Copyright (C) 2005-2006 by Jens Lukowski
 *   jens.lukowski@fh-swf.de
 *
 *   This file contains the implementation of the Printersetup GUI
 *
 *   PLEASE NOTICE: If not stated otherwise, whenever we speak of 'printer
 *                  drivers' or simply 'drivers', we mean the PPD files
 *                  and not the actual drivers.
 *
 *   $Id: guiprintermanualsetup.cpp,v 1.9 2006/04/18 19:35:03 hbci Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ***************************************************************************/

// App includes
#include "guiprintermanualsetup.h"
#include <printersetupdb.h>
#include "globals.h"

// Qt includes
#include <qmessagebox.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qdom.h>
#include <qapplication.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qregexp.h>
#include <qvalidator.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qinputdialog.h>


/**
 * Constructs a new printer manual setup dialog.
 *
 * @see QWidget for parameter documentation
 */
GuiPrinterManualSetup::GuiPrinterManualSetup( QWidget* parent, const char* name )
    : GuiPrinterManualSetupBase( parent,name )
{
  m_db                      = new PrinterSetupDb( LPS_DB_DOCUMENT_NAME );
  m_translationRecommended  = tr( "recommended", "This will be a appended to any recommended printer" );

  connect( pushButton_back, SIGNAL(clicked() ), this, SIGNAL( back() ) );
}


/**
 * This destructor 'takes care' of manually allocated memory.
 */
GuiPrinterManualSetup::~GuiPrinterManualSetup()
{
  if ( m_db )
    delete m_db;

  qDebug("Destructor GuiPrinterManualSetup finished");
}


/**
 * This method en-/disables widgets according to the appropriate bitmask.
 *
 * @param wf          any combination of \c WidgetFlag flags
 * @see   WidgetFlag
*/
void GuiPrinterManualSetup::enableWidgets( int wf )
{
  // list boxes
  listBox_models->setEnabled(             wf & ModelsFlag );
  listBox_drivers->setEnabled(            wf & DriversFlag );
  listBox_manufacturers->setEnabled(      wf & ManufacturersFlag );

  // labels
  label_drivers->setEnabled(              wf & DriversFlag );
  label_models->setEnabled(               wf & ModelsFlag );
  label_manufacturers->setEnabled(        wf & ManufacturersFlag );

  // combo boxes
  comboBox_ports->setEnabled(             wf & PortsFlag );

  // push buttons
  pushButton_driverDetails->setEnabled(   wf & DetailsFlag );
  pushButton_saveSettings->setEnabled(    wf & SaveSettingsFlag );
}


/**
 * This slot is called if an item in listBox_manufacturers is highlighted.
 * It fills the listBox_models with the associated models outof the database.
 *
 * @param item  The highlighted item in listBox_manufacturers.
 */
void GuiPrinterManualSetup::on_listBox_manufacturers_highlighted( QListBoxItem * item )
{
  if ( !item ) // If the manufacturer is not highlighted correctly
    return;

  listBox_models->clear();
  listBox_drivers->clear();

  // Get the Models to the highlightet manufacturer from the db
  QDomNodeList models = m_db->getModels( item->text() );

  // Fill the listBox_models
  for ( uint i=0; i<models.length(); i++ )
  {
    QDomElement modelElement = models.item( i ).toElement();

    if ( !modelElement.isNull() )
    {
      listBox_models->insertItem( modelElement.attribute( "name" ) );
    }
  }

  // No models in listBox_models?
  if ( listBox_models->count() <= 0 )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText( tr( "No model found for this manufacturer." ));
    enableWidgets( AllFlags ^ SaveSettingsFlag ^ DriversFlag ^ DetailsFlag ^ ModelsFlag );
  }

  enableWidgets( AllFlags ^ PortsFlag ^ DriversFlag ^ DetailsFlag ^ SaveSettingsFlag );

  label_status->setPaletteBackgroundColor( Qt::yellow );
  label_status->setText( tr( "Please choose a model now." ) );
}


/**
 * This slot is called if an item in listBox_models is highlighted.
 * It fills the listBox_drivers with the associated drivers outof the database.
 *
 * @param item  The highlighted item in on_listBox_models.
 */
void GuiPrinterManualSetup::on_listBox_models_highlighted( QListBoxItem * item )
{
  // Get the manufacturer out of the listBox_manufacturers
  QListBoxItem * selectedManufacturerItem = listBox_manufacturers->selectedItem();

  // Is the model item correctly highlighted
  if ( !item || !selectedManufacturerItem )
    return;

  listBox_drivers->clear();


  // Get the drivers from the database
  QDomNodeList drivers = m_db->getDriversByModel( selectedManufacturerItem->text(), item->text()  );


  for ( uint i=0; i<drivers.length(); i++ )
  {
    QDomElement driverElement = drivers.item( i ).toElement();

    if ( !driverElement.isNull() )
    {
      //replace "recommended" for the user. This must be undone later for selecting the driver.
      listBox_drivers->insertItem( driverElement.attribute( "nickName" ).replace( "recommended", m_translationRecommended ) );

      uint count = listBox_drivers->count();

      // If the driver is recommended, highlight it!
      if ( listBox_drivers->text( count - 1 ).contains( m_translationRecommended ) )
      {
        listBox_drivers->setCurrentItem( count - 1 );
        enableWidgets( AllFlags );
      }
    }
  }

  // No driver in drivers list box?
  if ( listBox_drivers->count() <= 0 )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText( tr( "No driver found for this model." ));
    enableWidgets( AllFlags ^ SaveSettingsFlag  ^ DriversFlag ^ DetailsFlag );
  }
  // No item selected? Then select the first one!
  else if( !listBox_drivers->selectedItem() )
  {
    enableWidgets( AllFlags ^ SaveSettingsFlag ^ PortsFlag );
    listBox_drivers->setCurrentItem( 0 );
  }

  lineEdit_printername->setText( item->text().replace( " ", "_" ) );
}


/**
 * This slot is called if an item in listBox_drivers is highlighted.
 * It fills the comboBox_ports with the on this machine founded ports to
 * connect a printer.
 *
 * @param item  The highlighted item in listBox_drivers.
 */
void GuiPrinterManualSetup::on_listBox_drivers_highlighted( QListBoxItem * item )
{
  // Is the driver item correctly highlighted (prevents clicking in nirvana)
  if ( !item )
    return;

  label_status->setPaletteBackgroundColor( Qt::yellow );

  // No port selected!
  if( comboBox_ports->currentText() == "" )
  {
    enableWidgets(AllFlags ^ SaveSettingsFlag);
    label_status->setPaletteBackgroundColor( Qt::yellow );
    label_status->setText( tr( "Driver:" ) +" " + item->text() + "<br>" +
                           tr( "Port:" ) + " " + tr( "Not yet selected." ) );
  }
  // Port selcted
  else
  {
    enableWidgets(AllFlags);
    label_status->setPaletteBackgroundColor( Qt::green );
    label_status->setText( tr( "Driver:" ) +" " + item->text() + "<br>" +
                           tr( "Port:" ) + " " + comboBox_ports->currentText() );
  }
}


/**
 * This slot calls the right help page for manual printer setup.
 */
void GuiPrinterManualSetup::on_pushButton_help_clicked()
{
  emit helpRequest( "printerManualSetup.html" );
}


/**
 * This slot is called, when the user clicks on the "Save Settings"
 * button. Then the current printer configuration is saved and the user may
 * decide whether to print a test page or not.
 *
 * @throws LpsNotice  if no printername is given.
 * @throws LpsError   if the driver element is null.
 */
void GuiPrinterManualSetup::on_pushButton_saveSettings_clicked()
{
  try
  {
    // If no printername is given throw exception.
    if ( lineEdit_printername->text() == "" )
      throw LpsNotice( tr( "Printer is unnamed. Please type in a name." ) );

    //undo the translation, because the driver name is untranslated in list.
    QString driverName = listBox_drivers->selectedItem()->text().replace( m_translationRecommended, "recommended" );

    // Get driver element
    QDomElement driver = m_db->getDriverByModel(
                           listBox_manufacturers->selectedItem()->text(),
                           listBox_models->selectedItem()->text(),
                           driverName );

    // If no driver element is null throw exception.
    if ( driver.isNull() )
      throw LpsError( "The driver element was NULL." );

    // Build command to be executed and run it
    QString printerName = lineEdit_printername->text();
    QString deviceUri   = comboBox_ports->currentText();
    QString ppdFilePath = driver.attribute( "ppdFilePath" ); // -p

    // Run lpadmin to install printer
    installPrinter( this, printerName, deviceUri, ppdFilePath );

    // Ask the user if a testpage shall be printed
    testPrintRequest( this, printerName );

    label_status->setText( "<p>" + tr( "Your printer has been set up successfully." ) + "</p>");
    label_status->setPaletteBackgroundColor( Qt::green );
  }
  catch ( LpsNotice e )
  {
    label_status->setPaletteBackgroundColor( Qt::yellow );
    label_status->setText( e );
    return;
  }
  catch ( LpsError e )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText( e );
    return;
  }

  label_status->setText( tr( "<p>Your printer has been configured successfully!</p>" ) );
  label_status->setPaletteBackgroundColor( Qt::green );
}


/**
 * Shows the widget and its child widgets.
 * This method might take a while until it returns because the XML
 * printer driver database is loaded.
 *
 *@throws LpsError  if the xml db could not been opened
 *@throws LpsError  if the content of the databes could not been set
 */
void GuiPrinterManualSetup::show()
{
  GuiPrinterManualSetupBase::show(); // Superclass Constructor

  emit showSmallHelp( "smallPrinterManualSetup.html" );
  label_status->setPaletteBackgroundColor( Qt::yellow );
  label_status->setText(tr( "Loading database..." ));
  setCursor( Qt::WaitCursor );
  setCaption( tr( "Manual Printer Setup" ) );
  qApp->processEvents();

  enableWidgets( NoneFlag | ManufacturersFlag );

  // Only valid characters for printername
  QRegExp rx( "[a-zA-Z_0-9]+" );
  QValidator * validator = new QRegExpValidator( rx, this );
  lineEdit_printername->setValidator( validator );

  // Clear all list boxes
  listBox_drivers->clear();
  listBox_models->clear();
  listBox_manufacturers->clear();

  // Set the default printername
  lineEdit_printername->setText( LPS_PRINTER_NAME );
  QString fileName= MP_DIR() + MONEYPENNY_DATA_DIR + LPS_DB_FILE;
  QFile file( fileName );

  try
  {
    if ( !file.open( IO_ReadOnly ) )
      throw LpsError( tr( "Could not open the database file." ) );
    if ( !m_db->setContent( &file ) )
      throw LpsError( tr( "Could not set content of xml database." ) );
  }
  catch ( LpsError e )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText( e );
    qDebug( e.latin1() );
    enableWidgets( NoneFlag );
  }

  // Fill the manufacturers list box
  QDomNodeList manufacturers = m_db->getManufacturers();
  for ( uint i=0; i<manufacturers.length(); i++ )
  {
    QDomElement manufacturersElement = manufacturers.item( i ).toElement();
    if ( !manufacturersElement.isNull() )
    {
      listBox_manufacturers->insertItem( manufacturersElement.attribute( "name" ) );
    }
  }

  // No manufacturers in listBox_manufacturers?
  if ( listBox_manufacturers->count() <= 0 )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText( tr( "No manufacturers found." ) );
    enableWidgets( NoneFlag );
  }
  else
  {
    label_status->setPaletteBackgroundColor( Qt::yellow );
    label_status->setText( tr( "Please select your printer's manufacturer name from the manufacturer list box." ) );
  }

  readPorts();
  unsetCursor();
}


/**
 * This slot is called if comboBox_ports is activated.
 * It enables the pushButton_saveSettings.
 *
 * @param itemtext  The selected text in the comboBox_ports as QString.
 */
void GuiPrinterManualSetup::on_comboBox_ports_activated( const QString & itemtext )
{
  // Is no port selected / activated?
  if ( itemtext == "" )
  {
    enableWidgets( AllFlags ^ SaveSettingsFlag );
  }
  else
  {
    enableWidgets( AllFlags );
    label_status->setPaletteBackgroundColor( Qt::yellow );
    label_status->setText( tr( "To save these settings and create the printer click on Save settings." ) );
  }
}


/**
 * This slot is called when the user clicks on pushButton_driverDetails
 * ("More Driver Information"). It opens a popup with details about the
 * selected driver.
 */
void GuiPrinterManualSetup::on_pushButton_driverDetails_clicked()
{
  try
  {
    showPrinterDriverDetails( listBox_drivers->selectedItem()->text().replace( m_translationRecommended, "recommended" ) );
  }
  catch ( LpsError e )
  {
    label_status->setText( e );
    label_status->setPaletteBackgroundColor( Qt::yellow );
  }
}


/**
 * Reads the ports using lpinfo and puts the results into comboBoxPorts.
 *
 * @throws LpsError if the ports could not be read
 */
void GuiPrinterManualSetup::readPorts()
{
  FILE * lpinfo = NULL;
  comboBox_ports->clear();

  lpinfo = popen( sudo() + "/usr/sbin/lpinfo -v", "r" );

  QTextStream* stream = NULL;

  try
  {
    if ( !lpinfo )
    {
      throw LpsError( tr( "Could not read ports" ) );
    }
    stream = new QTextStream(lpinfo,IO_ReadOnly);
    QString line;

    while( !stream->atEnd() )
    {
      line = QString( stream->readLine() );

      if ( !line.contains( "network" ) )
        comboBox_ports->insertItem( line.remove( "direct " ) );
    }
  }
  catch ( LpsError e )
  {
    label_status->setPaletteBackgroundColor( Qt::red );
    label_status->setText(e);
    qDebug( e.latin1() );
  }

  //
  // Never change the order of these deletions/closings!!!
  //
  if ( stream )
    delete stream;
  if ( lpinfo )
    pclose( lpinfo );

  comboBox_ports->setCurrentItem( 0 );
}
