/***************************************************************************
                          hobit.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "hobit.h"
#include "keyfilerdh2.h"
#include "keyfileopenhbci.h"
#include "hobit_transfertemplate.h"

#include <qcursor.h>
#include <qfile.h>
#include <qdir.h>
#include <qcstring.h>
#include <qvalidator.h>
#include <qinputdialog.h>
#include <qmessagebox.h>

#include <openhbci2/hbci.h>
#include <openhbci2/auth.h>
using namespace HBCI;


/* Default path for bank data */
#define BANKDATA_PATH "/usr/local/share/ktoblzcheck"

HoBIT::HoBIT(bool readonly,
             QWidget *parent,
             const string &openhbcifile,
             const string &aqmoneyfile,
             const string &templatefile,
             const string &hbcidatadir,
             const string &configdatadir,
             const string &applicationname,
             HoBIT_Connector * connector)
{     
     _readonly = readonly;
     _parent = parent;  
     _openhbcifile = openhbcifile; 
     _aqmoneyfile = aqmoneyfile; 
     _templatefile = templatefile;
     _hbcidatadir =  hbcidatadir;
     _ConfigDataDir = configdatadir;
     _applicationname = applicationname;
     _Connector = connector;
     _AdminJobFactory = NULL;
     _AccountJobFactory=NULL;  
     _Database = NULL;  

     cerr << "HbciDataDir is: " << _hbcidatadir << "\n";
     cerr << "OpenHbciFile is: " << _openhbcifile << "\n";
     cerr << "AqMoneyFile is: " << _aqmoneyfile << "\n";
     cerr << "TemplateFile is: " << _templatefile << "\n";
           
     createApi();
     qDebug("Constructor HoBIT"); 
}


HoBIT::~HoBIT(){
  //removeConfigDataDir();
  if(_AdminJobFactory)
    delete _AdminJobFactory;
  if(_AccountJobFactory)  
    delete _AccountJobFactory;
  if(_Database)  
    delete _Database;
  qDebug("Destructor HoBIT finished");
}


/**
*  executes both job queues for account and admin
*   jobs.
*/
HobitError HoBIT::executeJobQueue(){
      HobitError result = HobitErrorSuccess;      
      Error err; //hbci error
      Pointer<Outbox> queue;
      Pointer<Medium> medium;
      list<Pointer<User> > users = listUsers();
      Pointer<User> user = (*users.begin());
      
      if(user.isValid()){ //adjust the _SecirityMode of the interactor , if possible
        medium = user.ref().medium();
        if(medium.isValid())
        _interactor.ref().setSecurityMode(medium.ref().securityMode());
      }
      //check for a working internet connection
      QString url = getBankServer( MONEYPENNY_DEFAULT_COUNTRY, "*" );
      _Connector->setUrl(url);
      qDebug("\nServer: %s", url.latin1());
      _Connector->connectTCP();
      ConnectionState state = _Connector->connectionState();
      
      
      if(state == ConnectionConnected){
      try {
       DBG_INFO(0, "Creating new outbox");
       queue=new Outbox();

       _AdminJobFactory->updateJobQueue(queue.ptr());
       _AccountJobFactory->updateJobQueue(queue.ptr());  
       

        DBG_INFO(0, "Executing queue");
        err=_api.ref().executeQueue(queue);
         if (!err.isOk()) {
          cerr << "Error: " << err.errorString().c_str() << endl;
          result = HobitErrorExecution;
    }
    
    DBG_INFO(0, "Queue executed.");

    DBG_INFO(0, "Committing results");

    err=_api.ref().processOutboxResponse(queue.ref().response());
       if (!err.isOk()) {
              DBG_ERROR(0, "Error: %s", err.errorString().c_str());
              return HobitErrorExecution;
       }

    DBG_INFO(0, "Ok.");

    cerr << "Detailed job results:\n";

     if(_AdminJobFactory->showJobResults() != HobitErrorSuccess)
       result = HobitErrorExecution;
     if(_AccountJobFactory->showJobResults()!= HobitErrorSuccess)
       result = HobitErrorExecution;	

    // Execution of JobQueue finished
     }
     catch (Error xerr) {
        emit logTextToShow(xerr.errorString().c_str());
        return HobitErrorExecution; // no closeConnection
     }
  }
  else result = HobitErrorNotConnected;
  if(result == HobitErrorSuccess)
    saveEnvironment();
  _Connector->closeConnection();
  return result;
}


/** base method for creating a new user */
HobitError HoBIT::createUser(const string &mediumname,
                             const string &mediumType,
                             bool readonly,
                             int country,
                             const string &instid,
                             int hbciVersion,
                             const string &userid,
                             const string &customerid,
                             const string &server)
{
   deleteAllBanks();
                                                                                           
   Error err;
   err = _Database->createUser
     (mediumname,
      mediumType,
      readonly,
      country,
      instid,
      hbciVersion,
      userid,
      customerid,
      server);
   
   if (!err.isOk() || !saveEnvironment ()){
       return HobitErrorSave;
       QMessageBox::critical(_parent, "Error", tr("Error while writing environment"));
   }
   return HobitErrorSuccess;
}


/** Loads the OpenHBCI environment when the Program is started */
bool HoBIT::loadEnvironment(){
    cerr << "Loading HBCI environment....\n";
    string configdir = QString(_ConfigDataDir + "/").latin1();
    Error err = _api.ref().loadEnvironment( configdir + _openhbcifile, configdir + _aqmoneyfile);

    if (!err.isOk()) {
      DBG_ERROR(0, "Error loading environment (%s)",
                err.errorString().c_str());
    return false;
   }
  
   err = _Database->readTransferTemplates(configdir + _templatefile);
   if (!err.isOk()) {
   DBG_ERROR(0, "Error loading templates (%s)",
             err.errorString().c_str());
   return false;
  }
  return true;
}


/** Saves the OpenHBCI environment and data  */
bool HoBIT::saveEnvironment(){
    
    // check if saving was possible
    cerr << "Writing environment.\n";
    string configDirName(QString(_ConfigDataDir + "/").latin1());
    
    Error err = _api.ref().saveEnvironment(configDirName + _openhbcifile, configDirName + _aqmoneyfile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return false;
    }
  
    // check if saving was possible
    cerr << "Writing Templates.\n";
    err = _Database->saveTransferTemplates(configDirName + _templatefile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving templates (%s)",
                err.errorString().c_str());
      return false;
    }
  
    return true;
}



/** shows the logwindow and adds a logtext*/
void HoBIT::showLogText(QString string){
  emit logTextToShow(string);
}


/** creates a new security medium using the given information 
    * @ param mtype the medium type
    * @ param readonly decides whether the medium will be mounted readonly or not
    * @ param country the countrycode of the bank
    * @ param bankId the bankcode (bankleitzahl) of the institute
    * @ param userId the userId
    * @ param name the path of the new security file
    */
  HobitError HoBIT::createNewMedium  
   ( const string & mtype, 
     bool readonly, 
     int country, 
     const string & bankId, 
     const string & userId, 
     const string & keyname,
     const QString & newPassword)
{
    Pointer<Medium> md;
    try{	
      if(mtype == "RDHFile")
        _interactor.ref().setSecurityMode(HBCI_SECURITY_RDH);	
      else
        _interactor.ref().setSecurityMode(HBCI_SECURITY_DDV);	
        _interactor.ref().setNewPassword(newPassword); //give it to the interactor which is called fram aqmoney
        md = _api.ref().createNewMedium(mtype, readonly,country, bankId, userId, keyname);
    }
    catch(...){
      qDebug("Error when trying to create a new medium.");
    }
    
    if (!md.isValid()) return HobitErrorExecution;
    else{ 
        return HobitErrorSuccess;
    }
	
}


  //-------------------------------------------------------------------------
  //  DB-Access Operations
  //-------------------------------------------------------------------------

  /** Returns a std::list which includes all registered users as a struct
    */
  list<Pointer<User> > HoBIT::listUsers(){
  return _api.ref().getUsers();
  }


  /** removes all existing banks from the gwen database */
  void HoBIT::deleteAllBanks(){
      _Database->deleteAllBanks();    
  }

  /** searches the api for a special bank */
  Pointer<HBCI::Bank> HoBIT::findBank(int country, const string& instId){
    return _api.ref().findBank(country, instId);
  }
  
  

  /** changes the used hbci version */
  bool HoBIT::changeHbciVersion(int val){
      bool ok = _Database->changeHbciVersion(val);//This does not work!! messageengine has to be renewed! WR

      if (ok)
       saveEnvironment();
      return ok;
  }
  
  
  /** creates a new account with the overgiven values
  * this procedure will for example be done, when 
  * the retrieving of the account list from the 
  * bank server fails 
  *
  * @param name: Name of the account owner
  * @param accountId: the account number
  * @param accountSubId: Hbci allows subids for accounts 
  * 			 this parameter is normaly not needed
  * @param accountName:  you can give your account an description
  *
  */		
HobitError HoBIT::addAccount(const string& name, const string &accountId, const string &accountSubId, const string& accountName){

 bool saved = false;

 showLogText(tr("Creating account"));
 HobitError err = _Database->addAccount(name, accountId, accountSubId, accountName);
 if (err != 0) 
    showLogText(tr("Your input was not correct - aborting."));
 else {
    showLogText(tr("Saving data"));
    saved = saveEnvironment();
    if (!saved) 
       showLogText(tr("An error occured while saving data."));
    else showLogText(tr("Data have been saved."));
 }
 
 showLogText("\n");  
 if (err == 0 && saved) 
    showLogText(tr("Your account has been created successfully."));
 else if (err != 0) 
        showLogText(tr("Error creating your account"));
 else if (!saved) 
        showLogText(tr("Error while saving the environment"));

 return err;
}
  
  
    /** changes the account description */
  void HoBIT::changeAccountName(const string& accid, const string& name){
  _Database->changeAccountName(accid, name);
  saveEnvironment();
  }

  
  /** get a list of all accounts */
  list<Pointer<Account> > HoBIT::listAccounts(){
     return _api.ref().getAccounts();
  }
  
  
  /** searches the aqmapi for a single account
   */
  Pointer<Account> HoBIT::findAccount(const string& accid)
  {
    Pointer<Account> no;
    list<Pointer<Account> > accounts;
    list<Pointer<Account> >::iterator accit;
       
    accounts =  _api.ref().getAccounts(0,
                                       "*",
                                       accid,
                                       "*");

    if (accounts.size()< 1){
            cerr << "No matching accounts found\n";
            return no;
    }
                                       
    if (accounts.size()>1){
            cerr << "Problem in data - to many possible accounts\n";
            return no;
    }
    
    accit = accounts.begin();
    return *accit;	
  }

  /** Deletes account accountId by deleting its directory. (There is no deleteAccount function in openhbci) 
  */
  void HoBIT::deleteAccount(QString accountId){   
   QString accountDirName = _ConfigDataDir + "/" + "hbcidata/" + QString::number(MONEYPENNY_DEFAULT_COUNTRY);
   Pointer<Bank> bank= getBank();
   if(!bank.isValid())
    return;
   QString bankId = bank.ref().bankCode();
   accountDirName += "/" + bankId + "/accounts/" + accountId;
   QDir accountDir(accountDirName);
   if(accountDir.exists()){
      //delete account directory
      QString deleteCommand = "rm -R " + accountDirName;
      qDebug("HoBIT::deleteAccount: Deleting account directory with command: %s", deleteCommand.latin1());
      system(deleteCommand);
   }   
   string configdir = QString(_ConfigDataDir + "/").latin1();
   
   // remove account from moneypenny.conf
   QString accountStart = "[ \\t]+account \\{\\s*\\n\\s+char accountid=\"" + accountId + "\""; //skip the starting newline, it is needed in the file
   QString accountEnd = "\\s+\\} # account\\s*\\n"; //include the trailing newline
   QString confFileName = _ConfigDataDir + "/" + MONEYPENNY_AQMONEY_FILE;
   if(!removeTextBlock(accountStart, accountEnd, confFileName))
     return;
      
   //remove account from openhbci.conf
   accountStart = "[ \\t]+" + accountId + " \\{\\s*\\n"; //skip the starting newline         
   accountEnd = "\\s+\\} # "+ accountId + "\\s*\\n";
   confFileName = _ConfigDataDir + "/" + MONEYPENNY_OPENHBCI_FILE;
   if(!removeTextBlock(accountStart, accountEnd, confFileName))
      return;
      
     /* Error err = _api.ref().loadEnvironment( configdir + _openhbcifile, configdir + _aqmoneyfile); //reload data
      if(!err.isOk())
       qDebug("HoBIT::deleteAccount: Error when reloading data.");
     */  
   
  }

   /** returns a list of transactios  */
  list<Pointer <Transaction> > HoBIT:: listTransactions
    (int country,
     const string &bankCode,
     const string &accountId,
     const string &accountSubId,
     const QDate &fromDate,
     const QDate &toDate)
 {
  Date fromdate = Date(fromDate.day(), fromDate.month(), fromDate.year());
  Date todate = Date(toDate.day(), toDate.month(), toDate.year());
  return _Database->listTransactions
    (country,
     bankCode,
     accountId,
     accountSubId,
     fromdate, //HBCI date
     todate);  //HBCI date
 }

  
list <Pointer <DatedTransfer> >  HoBIT::listDatedTransfers
(int country,
 const string &bankCode,
 const string &accountId,
 const string &accountSubId)
{                                                                                    
    return _Database->listDatedTransfers(country, bankCode, accountId, accountSubId);
}


list <Pointer <StandingOrder> >  HoBIT::listStandingOrders
 ( int country,
   const string &bankCode,
   const string &accountId,
   const string &accountSubId)
{

    return _Database->listStandingOrders(country, bankCode, accountId, accountSubId);

}


/** generates the customers iniletter
*/
string HoBIT::generateIniLetter(){
     return _Database->generateIniLetter(0, "*", "*");
}


/** checks and returns the date of the last transaction stored
  * !!!!WARNING: This date may be in the future!!!!
  */
QDate HoBIT::getLastDateOfTransactions(){
    return _Database->getLastDateOfTransactions();
}


/**
  * returns a list of all known banks. Banks are 
  * returned as HoBIT_Bank-objects. 
  */
  list<HoBIT_Bank > HoBIT::listOfAllKnownBanks(){
      if( _banklist.empty()){  
         QString filename = MP_DIR()  + QString(MONEYPENNY_DATA_DIR) + "/"+ QString(MONEYPENNY_ALL_BANKDATA_FILE);
         _banklist = _Database->readBankFile(filename.latin1()); 
      } 
      return _banklist;   
}

/**
  * returns a list of all known banks, that support HBCI. Banks are 
  * returned as HoBIT_Bank objects. Reads and updates bank files automatically.
  */
  list<HoBIT_Bank> HoBIT::listOfKnownHbciBanks(){
     if(_hbcibanklist.empty()){ //not yet read, read now
       QString filename = MP_DIR()  + QString(MONEYPENNY_DATA_DIR) + "/"+ QString(MONEYPENNY_OLD_HBCI_BANKDATA_FILE);
       QFile file(filename);
       if(file.exists()){ //evaluate csv file and create the fast binary hbd format file
         _hbcibanklist = _Database->readBankFile(filename.latin1());
         filename = MP_DIR()  + QString(MONEYPENNY_DATA_DIR) + "/" + QString(MONEYPENNY_HBCI_BANKDATA_FILE);
         file.setName(filename); 
         bool ok = true;
         if(!file.open(IO_WriteOnly))
          QString rootPassword = QInputDialog::getText( tr("Information"),
           tr("Can not update bank data file. No write access. \nEnter root password to let MoneyPenny try to write updated bankdata file."),
            QLineEdit::Password,
            0,
            &ok);
         if(ok){ //try to write  
              ok = _Database->writeHbdBankFile(filename.latin1(), _hbcibanklist);
         }
         if(!ok){
             QMessageBox::critical(_parent, tr("Unrecoverable Error"), tr("Writing bankdata failed. Aborting now."));
             exit(1);
         }            
        _hbcibanklist = _Database->readBankFile(filename.latin1()); 
       }//no csv file, read hbd file
       else {
          filename = MP_DIR()  + QString(MONEYPENNY_DATA_DIR) + "/" + QString(MONEYPENNY_HBCI_BANKDATA_FILE);
          file.setName(filename); 
          _hbcibanklist = _Database->readBankFile(filename.latin1());
       }
     }// end list is empty
     return _hbcibanklist;     
}


 /** returns the hash fingerprint of the institute after resceiving the bank servers public keys 
 */
  string HoBIT::getBankFingerPrint(int country, const string& bankcode, const string& customerId){
    return _Database->getBankFingerPrint();
  }

  
    /** returns the fingerprint of the current keypair*/
  string HoBIT::getMyOwnFingerPrint(int country, const string& bankcode, const string& customerId){
    return _Database->getMyOwnFingerPrint();
  }
  
  
  /** returns all transfer templates */
  list<Pointer<HoBIT_TransferTemplate> > HoBIT::getTransferTemplates(){
     return _Database->getTransferTemplates();
  }
  
  
/** adds a transfer template to database
*/
void HoBIT::addTransferTemplate(HoBIT_TransferTemplate * transferTemplate){
   
   Pointer<HoBIT_TransferTemplate> temp = transferTemplate;
   _Database->addTransferTemplate(temp);
   saveEnvironment();
}


 /** creates a new transfer template 
*/
void HoBIT::deleteTransferTemplate(const string& ownAccountId,
                                   const string& otherName, 
                                   const string& otherAccountId,
                                   const string& otherBank,
                                   double value,
                                   const QStringList& purpose)
{
   _Database->deleteTransferTemplate( ownAccountId,
                                      otherName, 
                                      otherAccountId,
                                      otherBank,
                                      value,
                                      purpose);
   saveEnvironment();
}
  

//-------------------------------------------------------------------------
//  AdminJobs
//-------------------------------------------------------------------------

/** returns the account job queue */
  list<Pointer<AdminJob> > HoBIT::getAdminJobQueue(){
    return _AdminJobFactory->queue();
  }
  
  
  /** deleteAdminjob */
  void HoBIT::deleteAdminJob(int number){
    _AdminJobFactory->deleteJob(number);
  }

  
/** creates a job get keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetKeys(){
 HobitError err = _AdminJobFactory->createJobGetKeys();
 if (err == HobitErrorSuccess) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get keys, which will be executed directly */
HobitError HoBIT::createJobGetKeys(){
 Pointer <Medium> medium = getMedium();
 if (!medium.isValid() || medium.ref().securityMode() != HBCI_SECURITY_RDH) 
   return HobitErrorInvalidMediumType;
      
 return _AdminJobFactory->createJobGetKeys();
}


/** creates a job get systemid, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetSystemId(){
 HobitError err = _AdminJobFactory->createJobGetSystemId();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get systemid, which will be executed directly */
HobitError HoBIT::createJobGetSystemId(){
 return _AdminJobFactory->createJobGetSystemId();
}


/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetAccounts(){
 HobitError err = _AdminJobFactory->createJobGetAccounts();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get systemid, which will be executed directly */
HobitError HoBIT::createJobGetAccounts(){
 return _AdminJobFactory->createJobGetAccounts();
}



/** creates a job send keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobSendKeys(){
  HobitError err = _AdminJobFactory->createJobSendKeys();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobSendKeys(){
 Pointer <Medium> medium = getMedium();
 if (!medium.isValid() || medium.ref().securityMode() != HBCI_SECURITY_RDH) 
   return HobitErrorInvalidMediumType;
 return _AdminJobFactory->createJobSendKeys();
}


/** creates a job send keys, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetReports(){
 HobitError err = _AdminJobFactory->createJobGetReports();
 if (err == 0) 
   _AdminJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobGetReports(){
 return _AdminJobFactory->createJobGetReports();
}



//-------------------------------------------------------------------------
//  Account Jobs
//-------------------------------------------------------------------------

/** returns the account job queue */
 list<Pointer<AccountJob> > HoBIT::getAccountJobQueue(){
  return _AccountJobFactory->queue();
 }


/** deleteAccountjob */
  void HoBIT::deleteAccountJob(int number){
   _AccountJobFactory->deleteJob(number);
  }

  
  
  
/** creates a job get balance, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetBalance(){
 HobitError err = _AccountJobFactory->createJobGetBalance();
 if (err == 0) _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job send keys, which will be executed directly */
HobitError HoBIT::createJobGetBalance(){
 return _AccountJobFactory->createJobGetBalance();
}


/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetTransactions(Date fdate, Date tdate, const string& accountId){
  HobitError err = _AccountJobFactory->createJobGetTransactions(fdate, tdate, 0, "*", accountId);
 cerr << err;
 if (err == 0) 
    _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobGetTransactions(Date fdate, Date tdate, const string& accountId){
  return _AccountJobFactory->createJobGetTransactions(fdate, tdate, 0, "*", accountId);
}



/** creates a job get accounts, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetDatedTransfers(Date fdate, Date tdate, const string& accountId){
  HobitError err = _AccountJobFactory->createJobGetDatedTransfers(0, "*", accountId, fdate, tdate);
 cerr << err;
 if (err == 0) 
    _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobGetDatedTransfers(Date fdate, Date tdate, const string& accountId){
  if(! HoBIT_Bank::supportsJob(getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HITUBS"))
    return HobitErrorInvalidArgs; 
  return _AccountJobFactory->createJobGetDatedTransfers(0, "*", accountId, fdate, tdate);
}



/** creates a job get standing orders, which will be executed directly */
/*HobitError HoBIT::createAndSendJobGetStandingOrders(const string& accountId){
 HobitError err = _AccountJobFactory->createJobGetStandingOrders(0, "*", accountId);
 cerr << err;
 if (err == 0) 
   _AccountJobFactory->executeJobQueue();
 return err;
}
*/

/** creates a job get standing orders, which will be executed directly */
HobitError HoBIT::createJobGetStandingOrders(const string& accountId){
  if(!HoBIT_Bank::supportsJob(getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDABS"))  
    return HobitErrorInvalidArgs;
  return _AccountJobFactory->createJobGetStandingOrders(0, "*", accountId);  
}


/** creates a job get accounts, which will be executed directly */
HobitError HoBIT::createJobSingleTransfer
 (bool skipBad,
  int country,
  const string &bankCode,
  const string &accountId,
  const string &accountSubId,
  int otherCountry,
  const string& otherBankCode,
  const string& otherAccount,
  const list<string> &otherName,
  const list<string> &purpose,
  const  Value&  value,
  int textkey,
  Date date)
{
   return _AccountJobFactory->createJobSingleTransfer
    (skipBad,
     country,
     bankCode,
     accountId,
     accountSubId,
     otherCountry,
     otherBankCode,
     otherAccount,
     otherName,
     purpose,
     value,
     textkey,
     date);
}



/** Deletes a  dated transfer */
HobitError HoBIT::createJobDeleteDatedTransfer(Pointer<DatedTransfer> dt){
 if(!HoBIT_Bank::supportsJob(getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HITULS"))  
   return HobitErrorInvalidArgs;

list<string> purpose;
purpose.push_back("Purpose");

list<string> othername;
othername.push_back(dt.ref().otherName());

QString my = dt.ref().accountId().c_str();
QString other = dt.ref().otherAccountId().c_str();
QString id = dt.ref().jobId().c_str();

 return      _AccountJobFactory->createJobDeleteDatedTransfer
   ( false,                                     // skipbad
     dt.ref().country(),                        // country
     dt.ref().bankCode(),                   // bankcode
     (my.setNum(my.toInt())).ascii(),    // accountid
     "",                                           // accountSubId
     dt.ref().otherCountry(),           // other country
     dt.ref().otherBankId(),          // other bankcode
     (other.setNum(other.toInt())).ascii(),      // other account
     dt.ref().othernames(),           // other name
     dt.ref().purposes(),              // purpose
     dt.ref().value(),                  // value
     dt.ref().textKey(),               // textkey
     dt.ref().textKeyExt(),               // textkey
     dt.ref().date(),                   //date
     dt.ref().jobId());                  // jobid
}


  /** Deletes a  dated transfer */
HobitError HoBIT::createJobDeleteStandingOrder(Pointer<StandingOrder> sto){

if(!HoBIT_Bank::supportsJob(getConfigDataDir() + "/" + MONEYPENNY_OPENHBCI_FILE, "HIDALS"))  
  return HobitErrorInvalidArgs;
  
QString my = sto.ref().accountId().c_str();
QString other = sto.ref().otherAccountId().c_str();


showLogText(sto.ref().jobId().c_str());

return _AccountJobFactory->createJobDeleteStandingOrder
  ( false,
    sto.ref().country(),                            // country
    sto.ref().bankCode(),                      // bankcode
    (my.setNum(my.toInt())).ascii(),        // accountid
    sto.ref().otherCountry(),                    // other country
    sto.ref().otherBankId(),                     // other bankcode
    (other.setNum(other.toInt())).ascii(),    // other account
    sto.ref().othernames(),                      // other name
    sto.ref().purposes(),                          // purpose
    sto.ref().value(),                               // value
    sto.ref().textKey(),                            // textkey
    sto.ref().textKeyExt(),                        // textkey
    sto.ref().date(),                                 //date
    sto.ref().firstDate(),                            // first date
    sto.ref().lastDate(),                            // last date
    sto.ref().jobId(),                                // jobId
    sto.ref().turnus(),                              // turnus
    sto.ref().day(),                                 // day
    sto.ref().weekly());                           // weekly

}

 
  

/** creates a new standing Order */
        
HobitError HoBIT::createJobStartStandingOrder(bool skipBad,
                                                int country,
                                                const string &bankCode,
                                                const string &accountId,
                                                const string &accountSubId,
                                                int otherCountry,
                                                const string& otherBankCode,
                                                const string& otherAccount,
                                                const list<string> &otherName,
                                                const list<string> &purpose,
                                                const  Value&  value,
                                                int textkey,
                                                const Date& date,
                                                const Date& firstdate,
                                                const Date& lastdate,
                                                const int turnus,
                                                bool weekly,
                                                int day){

   return _AccountJobFactory->createJobStartStandingOrder
   (skipBad,
    country,
    bankCode,
    accountId,
    accountSubId,
    otherCountry,
    otherBankCode,
    otherAccount,
    otherName,
    purpose,
    value,
    textkey,
    date,
    firstdate,
    lastdate,
    turnus,
    weekly,
    day);   
}


/** Removes the data directory using a simple rm
 */ 
void HoBIT::removeUncryptedEnvironment()
{
   QString rm = "rm -r " + _ConfigDataDir;
   system(rm.latin1());
}


/** Returns the name of the bank for the given id. Returns an empty string, if no bank could be found.
*/
QString HoBIT::getBankNameForId(const QString & id){
  bool bankFound = false;
  QString result = "";
  list<HoBIT_Bank> banks = listOfAllKnownBanks();
  list<HoBIT_Bank>::iterator bit;
  for (bit = banks.begin(); bit != banks.end() && !bankFound; bit++)
  {
    QString bankId = (*bit).getBankId();

    if (bankId == id)
    {
      result = (*bit).getBankName();
      bankFound = true;
    }
  }//end for
  return result;
 }
 
/** Returns the name (url) of the bank server
*/
QString HoBIT::getBankServer(int country, const string &bankId){
      Pointer<Bank> bank = _api.ref().findBank(country,bankId);
      string url = bank.ref().address();
      return QString(url.c_str());
}


/**Creates the aqmoney engine
*/ 
void HoBIT::createApi(){          
     // setting up version
    _api = new AqMoneyAPI(_readonly);
    _api.ref().setSystemName(_applicationname);
    _api.ref().setSystemVersion(VERSION);
    _api.ref().setApplicationName(_applicationname + " " + VERSION);
       
  // creating and setting interactor and authentificator
    _interactor = new HoBIT_Interactor(_parent, _api.ptr());
    _auth = new HoBIT_Auth(_api.ptr(), _parent);
    _api.ref().setInteractor(_interactor.cast<Interactor>());
    _api.ref().setAuthentificator(_auth.cast<Auth>());
 
   // creating and setting members
   _Database = new DatabaseAccess(_api);
    
   // now setting directories and file names
   //top directory is crypt::_ConfigDataDir 
   _api.ref().setAapplicationDataDir(_ConfigDataDir + "/" + _hbcidatadir); //take care of Aa !!
  
   // creating JobFactories
   _AccountJobFactory = new AccountJobFactory(_api.ptr());
   _AdminJobFactory = new AdminJobFactory(_api.ptr());
              
   // create validator 
   _validator = new HoBIT_Validator(_api.ptr());
}


/** Returns the previously choosen customer */
Pointer<Bank> HoBIT::getBank(){

    list<Pointer<Bank> > banks;
    Pointer<Bank> bank;
           
    banks =  _api.ref().banks();
    
    if (banks.size()< 1){
            cerr << "No matching bank found\n";
            return bank;
    }
                                       
    if (banks.size()>1){
            cerr << "Problem in data - to many possible banks\n";
            return bank;
    }
    
    bank = *banks.begin();
    return bank;	
}

/** Returns the bankId BLZ
*/
QString HoBIT::getBankId(){
 Pointer<Bank> bank = getBank();
 QString bankId = bank.ref().bankCode();
 return bankId;
}

/** Import an RDH2 formatted key. Format is specified in "FinTS-Format der RDH2 Diskette" by PPI, Hamburg. see www.hbc-kerneli.org
*/
bool HoBIT::importRDH2Key(const QString & fileName, const QString & password){
  //Under construction!!!
  qDebug("\nFunction HoBIT::importRDH2Key");
 // KeyFileRDH2 * keyFileRDH2 = new KeyFileRDH2(fileName, password);
  
  KeyFileOpenHbci * keyFileOpenHbci = new KeyFileOpenHbci(fileName, password);
  if(!keyFileOpenHbci->exists())
    return false;
  keyFileOpenHbci->readFile();
  qDebug("\nFunction HoBIT::importRDH2Key: keyFile read");
  
  delete keyFileOpenHbci;
  qDebug("HoBIT::importRDH2Key");
  return true;  
    
}

/** Returns true for acceptable and intermediate accountId strings
*/
bool HoBIT::validateAccountIdString(const QString & accountId){
  // regular expression allows only digits from 0 to 9 with a length of 0 to 15
 QRegExp rx( "[0-9]{0,15}" );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 QString localAccountId = accountId;
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 return (v.validate(localAccountId, pos)==QValidator::Acceptable)  || (v.validate(localAccountId, pos)==QValidator::Intermediate);
}

/** Read property of AdminJobFactory m_AdminJobFactory. 
*/
AdminJobFactory * HoBIT::getAdminJobFactory()const{
	return _AdminJobFactory;
}
/** Read property of AccountJobFactory m_AccountJobFactory. 
*/
AccountJobFactory * HoBIT::getAccountJobFactory()const{
	return _AccountJobFactory;
}


/** returns the current HBCI mode */
HBCI_MODE HoBIT::getHbciMode(){
     list<Pointer<Customer> > custs=_api.ref().getCustomers();
     list<Pointer<Customer> >::iterator cit;
     Pointer<Medium> medium;
    
      if (custs.size() != 1) {
            cerr << "No or to many customers found. Number of customers: " << custs.size()<<"\n";
	    return HBCI_MODE_ERROR;
        }
      else {
        cit=custs.begin();
        medium=(*cit).ref().user().ref().medium();
	}

	if (medium.ref().securityMode() == HBCI_SECURITY_RDH) 
         return HBCI_MODE_RDH;
	else if (medium.ref().securityMode() == HBCI_SECURITY_DDV) 
           return HBCI_MODE_DDV;
	else return HBCI_MODE_ERROR;
}


/** Returns true, if no jobs are in job queues
*/
bool HoBIT::areJobQueuesEmpty(){
  list<Pointer<AccountJob> > accountjoblist = getAccountJobQueue();
  list<Pointer<AdminJob> > adminjoblist = getAdminJobQueue();

  return (accountjoblist.empty() && adminjoblist.empty());
}

/** Returns the user from OpenHbci database. Return value should be checked with isValid().
*/
Pointer<User> HoBIT::getUser(){
  Pointer<Bank> bank = getBank();;
  Pointer<User> user; //constructor creates an invalid Pointer
  list<Pointer<User> > users;
   
  if (bank.isValid()){  
    users = bank.ref().users();    
    if(users.size() > 0){
      user = *users.begin();
    }
  }
  return user;
}

/** Returns the currently used medium from OpenHbci database. Return value should be checked with isValid().
*/
Pointer<Medium> HoBIT::getMedium(){
   Pointer<Medium> medium;
   Pointer<User> user = getUser();
   if(user.isValid())
     medium = user.ref().medium();
   return medium; 
}

/** Returns the first customer from the user's customer list. Return value should be checked with isValid().
*/
Pointer<Customer> HoBIT::getCustomer(){
   Pointer<User> user = getUser();
   list<Pointer<Customer> > customers;
   Pointer<Customer> customer;
   if(user.isValid()){
     customers = user.ref().customers();     
     if (customers.size() > 0)        
       customer = *customers.begin();
   } 
   return customer; 
 }
 
/** Returns true, if cardreader is up.
*/
bool HoBIT::isCardReaderOk(){
 bool result = false;
 
 //check for an installed plugin
 string cardName = ""; //let the plugin create the name
 Pointer<MediumPluginList> pluginList =_api.ref().enumerateMediumPlugins();
 list< Pointer< MediumPlugin > > plugins = pluginList.ref().plugins();
 list< Pointer< MediumPlugin > >::iterator it; //this is the current plugin
 if(plugins.empty())
   result = false;
 else{
   for(it = plugins.begin(); it != plugins.end(); it++){// check for a plugin that supports cards
    if((*it).ref().mediumType() == MediumTypeCard){
       _interactor.ref().setSecurityMode(HBCI_SECURITY_DDV);
      if((*it).ref().mediumCheck(cardName).isOk()) //checks for an accessible card
       result = true;
    }  
   }//end for   
 }//end else
 if(!result){
     QMessageBox::critical(_parent, tr("Error"), 
    tr("Can not load ddvcard2 plugin. Check your libchipcard installation."));
    return result;
 } 
  result = isChipcardDaemonRunning();
 return result;
}

/** Returns true, if daemon is running. Else tries to start daemon with sudo. If this fails, returns false.
*/
bool HoBIT::isChipcardDaemonRunning(){
//check for a running chipcardd2 demon
 int returnValue = system("ps -A | grep chipcardd2"); //check for a running chipcarddd2 process
 if(returnValue != 0){
   bool okPressed;
   if(! USE_LIVE_CD_FEATURES){ //ask user for password
       QString rootPassword = QInputDialog::getText( tr("Information"),
         tr("Can not read chipcard. Driver daemon chipcardd2 is not yet running. \nEnter root password to let MoneyPenny try to start it."),
          QLineEdit::Password,
          0,
          &okPressed);
       if(okPressed){ //try to start daemon
        QString sudoCommand = "echo " + rootPassword + " | sudo -S /usr/sbin/chipcardd2 --pidfile /var/run/chipcardd2.pid --logfile /var/log/chipcardd2.log";
        system(sudoCommand.latin1());  
        system("sudo -K"); //kill the sudo timestamp         
        QMessageBox::information(_parent, tr("Information"), //this gives the daemon time to start, not very smart, but it works
        tr("MoneyPenny has started daemon with command: ") + "\n/usr/sbin/chipcardd2 --pidfile /var/run/chipcardd2.pid --logfile /var/log/chipcardd2.log");
       } 
       else return false; 
   }
   else { // no need for a password, try to start immediately
     QString sudoCommand = "sudo /usr/sbin/chipcardd2 --pidfile /var/run/chipcardd2.pid --logfile /ramdisk/home/knoppix/chipcardd2.log";
     qDebug("Daemon started using command: %s", sudoCommand.latin1());
     system(sudoCommand.latin1());  
     system("sudo -K"); //kill the sudo timestamp         
   }
   
   returnValue = system("ps -A | grep chipcardd2"); //check again for a running chipcarddd2 process
   if(returnValue != 0){ //still not ok, give up
    QMessageBox::critical(_parent, tr("Installation Error"), 
    tr("Starting daemon chipcardd2 failed. \nCheck your libchipcard2 installation"));
    qDebug("Starting chipcardd2 daemon failed.");
    return false;
   }
   else return true; //starting was successful
 }//end if returnValue 
 else return true; //daemon is in process list
}

/** write property for _CryptDir
*/
void HoBIT::setCryptDir(const string & newVal){
//  _CryptDir = newVal;
}

/** Removes all the text from the beginning of startBlock to the end of endBlock from file fileName.
*/
bool HoBIT::removeTextBlock(QString startBlock, QString endBlock, QString fileName){
      QFile confFile (fileName);
      qDebug("HoBIT::removeTextBlock: from File: %s  from Startblock: %s  to EndBlock: %s",
             fileName.latin1(), startBlock.latin1(), endBlock.latin1());
      if(!confFile.open(IO_ReadWrite)) //check for read and write 
       return false;
      QTextStream s (&confFile);
      QString confData = s.read();
      confFile.close();
      QFile::remove(fileName);
      QRegExp re;
      re.setMinimal(true); //switch off "greedy" behaviour
      re.setPattern(startBlock + ".*" + endBlock); //find the  textblock
      re.search(confData); //catch it
      qDebug("Captured Text: %s", re.capturedTexts()[0].latin1());
      confData.remove(re.capturedTexts()[0]); //remove it
      confFile.open(IO_ReadWrite);
      s << confData;
      confFile.close();
      return true;
}

/** Reads the data from the chipcard. There MAY be userid, bankid, server. 
*/
int HoBIT::readCardData(int & country, QString & bankId, QString & userId, QString & server, QString & cardnumber){
 _interactor.ref().setSecurityMode(HBCI_SECURITY_DDV);
 return _Database->readCardData(country, bankId, userId, server, cardnumber);
}


/** Read property of QString _CurrentAccount. 
*/
const QString& HoBIT::getCurrentAccount()const{
	return _CurrentAccount;
}

/** Read property of QString _CurrentAccountName. 
*/
const QString& HoBIT::getCurrentAccountName()const{
	return _CurrentAccountName;
}

/** Write property of QString _CurrentAccount. 
*/
void HoBIT::setCurrentAccount( const QString& _newVal){
   _CurrentAccount = _newVal;
   Pointer<Account> account;
   account = findAccount(_CurrentAccount.ascii());
   _CurrentAccountName = account.ref().accountName().c_str();
}

/** Returns temporary config data directory.
*/
QString  HoBIT::getConfigDataDir(){
  return _ConfigDataDir;
}

